package com.bytedance.transbridge;

import android.view.View;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.bytedance.fluttermk.FlutterAdapter;
import com.bytedance.fluttermk.MethodCallMk;
import com.bytedance.fluttermk.MethodChannelMk;
import com.bytedance.transbridge.codec.GsonMethodCodec;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.view.FlutterView;


public class FlutterAdapterImpl implements FlutterAdapter {
    public static class MethodChannelImpl extends MethodChannelMk.ChannelStub {
        private MethodChannel channel;

        public MethodChannelImpl(MethodChannel channel) {
            this.channel = channel;
        }

        @Override
        public void invokeMethod(@NonNull String method, Object args) {
            channel.invokeMethod(method, args);
        }

        @Override
        public void invokeMethod(@NonNull String method, Object args, final ResultMk resultMk) {
            channel.invokeMethod(method, args, new Result() {
                @Override
                public void success(Object o) {
                    resultMk.success(o);
                }

                @Override
                public void error(String s, String s1, Object o) {
                    resultMk.error(s, s1, o);
                }

                @Override
                public void notImplemented() {
                    resultMk.notImplemented();
                }
            });
        }

        @Override
        public void setMethodCallHandler(final MethodCallHandlerMk methodCallHandlerMk) {
            channel.setMethodCallHandler(new MethodCallHandler() {
                @Override
                public void onMethodCall(@NonNull final MethodCall methodCall, @NonNull Result result) {
                    methodCallHandlerMk.onMethodCall(new MethodCallStubImpl(methodCall), new ResultImpl(result));
                }
            });
        }
    }

    public static class ResultImpl extends MethodChannelMk.ResultStub {
        private Result mResult;

        public ResultImpl(Result mResult) {
            this.mResult = mResult;
        }

        @Override
        public void success(@Nullable Object result) {
            mResult.success(result);
        }

        @Override
        public void error(String errorCode, @Nullable String errorMessage, @Nullable Object errorDetails) {
            mResult.error(errorCode, errorMessage, errorDetails);
        }

        @Override
        public void notImplemented() {
            mResult.notImplemented();
        }
    }

    public static class MethodCallHandlerImpl extends MethodChannelMk.MethodCallHandlerStub {
        @Override
        public void onMethodCall(@NonNull MethodCallMk call, @NonNull MethodChannelMk.ResultMk result) {

        }
    }

    public static class MethodCallStubImpl extends MethodCallMk.MethodCallStub {
        private final MethodCall call;

        public MethodCallStubImpl(MethodCall call) {
            this.call = call;
        }

        @Override
        public String name() {
            return call.method;
        }

        @Nullable
        @Override
        public <T> T argument(String key) {
            return call.argument(key);
        }

        @Override
        public boolean hasArgument(String key) {
            return call.hasArgument(key);
        }

        @Override
        public Object arguments() {
            return call.arguments;
        }
    }

    @Override
    public MethodChannelMk createMethodChannel(String name, View view) {
        if (!(view instanceof FlutterView)) {
            throw new IllegalArgumentException("view should be a flutterview which has binarymessager's interface");
        }
        return new MethodChannelImpl(new MethodChannel((FlutterView) view, name, GsonMethodCodec.INSTANCE));
    }
}
