
package com.bytedance.flutter.vessel.transbridge.integrete;

import com.bytedance.flutter.vessel.utils.GsonUtils;
import com.bytedance.transbridge.codec.GsonMessageCodec;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;

import java.math.BigInteger;
import java.nio.ByteBuffer;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.flutter.plugin.common.FlutterException;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodCodec;
import io.flutter.plugin.common.StandardMessageCodec;

public final class GsonMethodCodec implements MethodCodec {
    public static final GsonMethodCodec INSTANCE = new GsonMethodCodec();
    public static final byte JSON_TYPE = 0;
    public static final byte NORMAL_TYPE = 1;

    private GsonMethodCodec() {
    }

    public ByteBuffer encodeMethodCall(MethodCall methodCall) {
        Map<String, Object> map = new HashMap<>();
        map.put("method", methodCall.method);
        map.put("args", methodCall.arguments);
        return GsonMessageCodec.INSTANCE.encodeMessage(map);
    }

    public MethodCall decodeMethodCall(ByteBuffer message) {
        Object json = GsonMessageCodec.INSTANCE.decodeMessage(message);
        if (json instanceof JsonElement) {
            JsonObject map = ((JsonElement) json).getAsJsonObject();
            String method = map.get("method").getAsString();
            JsonElement arguments = map.get("args");
            if (method != null && method.length() > 0) {
                return new MethodCall(method, arguments);
            }
        }

        throw new IllegalArgumentException("Invalid method call: " + json);
    }

    private boolean isPrimaryType(Object result) {
        return result instanceof int[]
                || result instanceof byte[]
                || result instanceof Boolean
                || result instanceof long[]
                || result instanceof double[]
                || result instanceof Map
                || result instanceof List
                || result instanceof String
                || result instanceof Number;
    }

    // native result => flutter.
    public ByteBuffer encodeSuccessEnvelope(Object result) {
        if (isPrimaryType(result)) {
            final ByteBuffer buffer = StandardMessageCodec.INSTANCE.encodeMessage(result);
            final ByteBuffer resultByte = ByteBuffer.allocate(buffer.array().length + 1);
            resultByte.put(0, NORMAL_TYPE);
            resultByte.put(buffer.array());
            return resultByte;
        }
        JsonArray resultArr = new JsonArray();
        resultArr.add(GsonUtils.toJsonElement(result));
        final ByteBuffer buffer =  GsonMessageCodec.INSTANCE.encodeMessage(resultArr);
        final ByteBuffer resultByte = ByteBuffer.allocate(buffer.array().length + 1);
        resultByte.put(0, JSON_TYPE);
        resultByte.put(buffer.array());
        return buffer;
    }

    public ByteBuffer encodeErrorEnvelope(String errorCode, String errorMessage, Object errorDetails) {
        JsonArray resultArr = new JsonArray();
        resultArr.add(errorCode);
        resultArr.add(errorMessage);
        resultArr.add(GsonUtils.toJsonElement(errorDetails));
        return GsonMessageCodec.INSTANCE.encodeMessage(resultArr);
    }

    // flutter result => native. 0: json 1: normal.
    public Object decodeEnvelope(ByteBuffer envelope) throws FlutterException {
        if (!envelope.hasRemaining()) {
            throw new IllegalArgumentException("Message corrupted");
        }
        final byte type = envelope.get();
        switch (type) {
            case JSON_TYPE: {
                Object json = GsonMessageCodec.INSTANCE.decodeMessage(envelope);
                if (json instanceof JsonArray) {
                    JsonArray array = (JsonArray) json;
                    if (array.size() == 1) {
                        return array.get(0);
                    }

                    if (array.size() == 3) {
                        Object code = array.get(0).getAsString();
                        Object message = array.get(1).getAsString();
                        Object details = array.get(2);
//                throw new FlutterException(code, message, details);
                    }
                }

                break;
            }
            case NORMAL_TYPE: {
                return StandardMessageCodec.INSTANCE.decodeMessage(envelope);
            }
        }
        throw new IllegalArgumentException("Invalid envelope: " + envelope.toString());
    }
}

