
package com.example.bridge_client_example.codec;

import com.example.bridge_client_example.utils.GsonUtils;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;

import java.io.ByteArrayOutputStream;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.flutter.plugin.common.FlutterException;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodCodec;
import io.flutter.plugin.common.StandardMessageCodec;


public final class GsonMethodCodec implements MethodCodec {
    public static final GsonMethodCodec INSTANCE = new GsonMethodCodec();
    public static final byte JSON_TYPE = 0;
    public static final byte NORMAL_TYPE = 1;

    private GsonMethodCodec() {
    }

    public static class MStandardMessageCodec extends StandardMessageCodec {
        @Override
        protected void writeValue(ByteArrayOutputStream stream, Object value) {
            super.writeValue(stream, value);
        }

        public Object readValueBy(ByteBuffer buffer) {
            return super.readValue(buffer);
        }

        static final class ExposedByteArrayOutputStream extends ByteArrayOutputStream {
            byte[] buffer() {
                return buf;
            }
        }
    }

    private ByteBuffer convertToStandardCodec(Object object) {
        MStandardMessageCodec.ExposedByteArrayOutputStream stream = new MStandardMessageCodec.ExposedByteArrayOutputStream();
        stream.write(NORMAL_TYPE);
        new MStandardMessageCodec().writeValue(stream, object);
        ByteBuffer buffer = ByteBuffer.allocateDirect(stream.size());
        buffer.put(stream.buffer(), 0, stream.size());
        return buffer;
    }

    public ByteBuffer encodeMethodCall(MethodCall methodCall) {
        Map<String, Object> map = new HashMap<>();
        map.put("method", methodCall.method);
        map.put("args", methodCall.arguments);
        // native call standard method.
        if (isPrimaryType(methodCall.arguments)) {
            return convertToStandardCodec(map);
        }

        return GsonMessageCodec.INSTANCE.encodeMessage(map);
    }

    public MethodCall decodeMethodCall(ByteBuffer message) {
        if (!message.hasRemaining()) {
            throw new IllegalArgumentException("Message corrupted");
        }

        message.order(ByteOrder.nativeOrder());
        final byte type = message.get();
        if (type == NORMAL_TYPE) {
            final MStandardMessageCodec messageCodec = new MStandardMessageCodec();
            final Map params = (Map) messageCodec.readValueBy(message);
            final Object method = params.get("method");
            final Object arguments = params.get("args");
            if (method instanceof String && !message.hasRemaining()) {
                return new MethodCall((String) method, arguments);
            }
            throw new IllegalArgumentException("Method call corrupted");
        } else {
            Object json = GsonMessageCodec.INSTANCE.decodeMessage(message.slice());
            if (json instanceof JsonElement) {
                JsonObject map = ((JsonElement) json).getAsJsonObject();
                String method = map.get("method").getAsString();
                JsonElement arguments = map.get("args");
                if (method != null && method.length() > 0) {
                    return new MethodCall(method, arguments);
                }
            }
            throw new IllegalArgumentException("Invalid method call: " + json);
        }
    }

    private boolean isPrimaryType(Object result) {
        return result instanceof int[]
                || result instanceof byte[]
                || result instanceof Boolean
                || result instanceof long[]
                || result instanceof double[]
                || result instanceof Map
                || result instanceof List
                || result instanceof String
                || result instanceof Number;
    }

    // native result => flutter.
    public ByteBuffer encodeSuccessEnvelope(Object result) {
        if (isPrimaryType(result)) {
            return convertToStandardCodec(result);
        }

        JsonArray resultArr = new JsonArray();
        resultArr.add(GsonUtils.toJsonElement(result));
        return GsonMessageCodec.INSTANCE.encodeMessage(resultArr);
    }

    public ByteBuffer encodeErrorEnvelope(String errorCode, String errorMessage, Object errorDetails) {
        JsonArray resultArr = new JsonArray();
        resultArr.add(errorCode);
        resultArr.add(errorMessage);
        resultArr.add(GsonUtils.toJsonElement(errorDetails));
        return GsonMessageCodec.INSTANCE.encodeMessage(resultArr);
    }

    // flutter result => native. 0: json 1: normal.
    public Object decodeEnvelope(ByteBuffer envelope) throws FlutterException {
        if (!envelope.hasRemaining()) {
            throw new IllegalArgumentException("Message corrupted");
        }
        final byte type = envelope.get();
        if (type == NORMAL_TYPE) {
            return StandardMessageCodec.INSTANCE.decodeMessage(envelope.slice());
        } else {
            Object json = GsonMessageCodec.INSTANCE.decodeMessage(envelope.slice());
            if (json instanceof JsonArray) {
                JsonArray array = (JsonArray) json;
                if (array.size() == 1) {
                    return array.get(0);
                }

                if (array.size() == 3) {
                    Object code = array.get(0).getAsString();
                    Object message = array.get(1).getAsString();
                    Object details = array.get(2);
//                throw new FlutterException(code, message, details);
                }
            }
        }
        throw new IllegalArgumentException("Invalid envelope: " + envelope.toString());
    }
}
